@@ -38,7 +38,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.9 2001/08/08 00:03:28 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.9.2.1 2001/10/13 04:20:24 shadow Exp $");
 
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
@@ -213,7 +213,7 @@ int afs_FlushVCache(struct vcache *avc, int *slept)
     /* This should put it back on the vnode free list since usecount is 1 */
     afs_vcount--;
     vSetType(avc, VREG);
-    if (avc->vrefCount > 0) {
+    if (VREFCOUNT(avc) > 0) {
         VN_UNLOCK((struct vnode *)avc);
         AFS_RELE((struct vnode *)avc);
     } else {
@@ -568,6 +568,10 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 	 int vmax = 2 * afs_cacheStats;
 	 int vn = VCACHE_FREE;
 
+         AFS_GUNLOCK();
+	 shrink_dcache_sb(afs_globalVFS);
+	 AFS_GLOCK();
+
 	 i = 0;
 	 for(tq = VLRU.prev; tq != &VLRU && vn > 0; tq = uq) {
 	    tvc = QTOV(tq);
@@ -582,7 +586,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 	    if (tvc == afs_globalVp)
 		continue;
 
-	    if ( tvc->vrefCount && tvc->opens == 0 ) {
+	    if ( VREFCOUNT(tvc) && tvc->opens == 0 ) {
 		struct inode *ip = (struct inode*)tvc;
 		if (list_empty(&ip->i_dentry)) {
 		    vn --;
@@ -656,10 +660,10 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 		 refpanic ("Exceeded pool of AFS vnodes(VLRU cycle?)");
 	    else if (QNext(uq) != tq)
 		 refpanic ("VLRU inconsistent");
-	    else if (tvc->vrefCount < 1) 
+	    else if (VREFCOUNT(tvc) < 1) 
 		 refpanic ("refcnt 0 on VLRU");
 
-	    if ( tvc->vrefCount == 1   &&   tvc->opens == 0 
+	    if ( VREFCOUNT(tvc) == 1   &&   tvc->opens == 0 
 		&& (tvc->states & CUnlinkedDel) == 0) {
 		code = afs_FlushVCache(tvc, &fv_slept);
 		if (code == 0) {
@@ -711,7 +715,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 
 #ifdef AFS_DARWIN_ENV
 	   if (tvc->opens == 0 && ((tvc->states & CUnlinkedDel) == 0) &&
-                tvc->vrefCount == 1 && UBCINFOEXISTS(&tvc->v)) {
+                VREFCOUNT(tvc) == 1 && UBCINFOEXISTS(&tvc->v)) {
                osi_VM_TryReclaim(tvc, &fv_slept);
                if (fv_slept) {
                   uq = VLRU.prev;
@@ -720,7 +724,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
                }
             }
 #endif
-	   if (tvc->vrefCount == 0 && tvc->opens == 0
+	   if (VREFCOUNT(tvc) == 0 && tvc->opens == 0
 	       && (tvc->states & CUnlinkedDel) == 0) {
 		code = afs_FlushVCache(tvc, &fv_slept);
 		if (code == 0) {
@@ -815,7 +819,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
     /* Hold it for the LRU (should make count 2) */
     VN_HOLD((struct vnode *)tvc);
 #else	/* AFS_OSF_ENV */
-    tvc->vrefCount = 1;	/* us */
+    VREFCOUNT_SET(tvc, 1);	/* us */
 #endif	/* AFS_OSF_ENV */
 #ifdef	AFS_AIX32_ENV
     LOCK_INIT(&tvc->pvmlock, "vcache pvmlock");
@@ -1081,7 +1085,7 @@ afs_FlushActiveVcaches(doflocks)
 		/*
 		 * That's because if we come in via the CUnlinkedDel bit state path we'll be have 0 refcnt
 		 */
-		osi_Assert(tvc->vrefCount > 0);
+		osi_Assert(VREFCOUNT(tvc) > 0);
 		AFS_RWLOCK((vnode_t *)tvc, VRWLOCK_WRITE);
 #endif
 		ObtainWriteLock(&tvc->lock,52);
@@ -1136,7 +1140,7 @@ afs_FlushActiveVcaches(doflocks)
 		AFS_FAST_RELE(tvc);
 		if (didCore) {
 #ifdef	AFS_GFS_ENV
-		    tvc->vrefCount--;
+		    VREFCOUNT_DEC(tvc);
 #else
 		    AFS_RELE((struct vnode *)tvc);
 #endif
@@ -1145,7 +1149,7 @@ afs_FlushActiveVcaches(doflocks)
 		}
 	    }	       
 #ifdef AFS_DARWIN_ENV
-            if (tvc->vrefCount == 1 && UBCINFOEXISTS(&tvc->v)) {
+            if (VREFCOUNT(tvc) == 1 && UBCINFOEXISTS(&tvc->v)) {
 		if (tvc->opens) panic("flushactive open, hasubc, but refcnt 1");
 		osi_VM_TryReclaim(tvc,0);
 	    }
@@ -1683,8 +1687,15 @@ loop:
     /* stat the file */
     afs_RemoveVCB(afid);
     {
-    struct AFSFetchStatus OutStatus;
-    code = afs_FetchStatus(tvc, afid, areq, &OutStatus);
+	struct AFSFetchStatus OutStatus;
+
+	if (afs_DynrootNewVnode(tvc, &OutStatus)) {
+	    afs_ProcessFS(tvc, &OutStatus, areq);
+	    tvc->states |= CStatd | CUnique;
+	    code = 0;
+	} else {
+	    code = afs_FetchStatus(tvc, afid, areq, &OutStatus);
+	}
     }
 
     if (code) {
@@ -2694,7 +2705,7 @@ void shutdown_vcache(void)
 		    vms_delete(tvc->segid);
 		    AFS_GLOCK();
 		    tvc->segid = tvc->vmh = NULL;
-		    if (tvc->vrefCount) osi_Panic("flushVcache: vm race");
+		    if (VREFCOUNT(tvc)) osi_Panic("flushVcache: vm race");
 		}
 		if (tvc->credp) {
 		    crfree(tvc->credp);
