@@ -16,7 +16,7 @@
 #include <afs/param.h>
 #endif
 
-RCSID("$Header: /cvs/openafs/src/auth/ktc.c,v 1.7 2001/08/08 00:03:37 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/auth/ktc.c,v 1.7.2.1 2001/10/13 04:20:47 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "../afs/sysincludes.h"
@@ -48,10 +48,20 @@ RCSID("$Header: /cvs/openafs/src/auth/ktc.c,v 1.7 2001/08/08 00:03:37 shadow Exp
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <afs/vice.h>
 #ifdef	AFS_AIX_ENV
 #include <sys/lockf.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "auth.h"
 #include <afs/venus.h>
 #include <afs/afsutil.h>
